#include "gcodeviewer.h"
#include "simpletextedit.h"
#include <QScrollArea>
#include <QPushButton>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QFileDialog>
#include <QFile>
#include <QScrollBar>
#include "gcoder.h"
#include "viewer.h"
#include "mesh.h"

GCodeViewer *GCodeViewer::pInstance = NULL;
GCodeViewer *GCodeViewer::instance()
{
    if (!pInstance)
        pInstance = new GCodeViewer;
    return pInstance;
}

GCodeViewer::GCodeViewer(QWidget *parent) :
    QDockWidget(parent)
{
	setObjectName("GCodeViewer");

    QWidget *widget = new QWidget;
    QVBoxLayout *layout = new QVBoxLayout;
    widget->setLayout(layout);

    QHBoxLayout *scroll_layout = new QHBoxLayout;
    scroll_bar = new QScrollBar(Qt::Vertical);
    gcode_view = new SimpleTextEdit;
    gcode_view->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    connect(scroll_bar,SIGNAL(valueChanged(int)),gcode_view,SLOT(setScrollPosition(int)));
    connect(gcode_view,SIGNAL(scrollPositionChanged(int)),scroll_bar,SLOT(setValue(int)));
    scroll_bar->setMinimum(0);
    scroll_bar->setMaximum(gcode_view->getMaxScroll());
    scroll_bar->setValue(0);
    scroll_layout->addWidget(gcode_view);
    scroll_layout->addWidget(scroll_bar);

    layout->addLayout(scroll_layout);

    QPushButton *bLoad = new QPushButton(QIcon::fromTheme("document-open"), tr("Load GCode"));
    connect(bLoad, SIGNAL(pressed()), this, SLOT(loadGCode()));
    QPushButton *bSave = new QPushButton(QIcon::fromTheme("document-save"), tr("Save GCode"));
    connect(bSave, SIGNAL(pressed()), this, SLOT(saveGCode()));
    QHBoxLayout *lButtons = new QHBoxLayout;
    lButtons->addWidget(bLoad);
    lButtons->addWidget(bSave);

    layout->addLayout(lButtons);

    setWidget(widget);
    setWindowTitle(tr("GCode"));
}

QByteArray GCodeViewer::getGCode() const
{
    return gcode_view->toPlainText();
}

void GCodeViewer::setGCode(const QByteArray &gcode)
{
    gcode_view->setPlainText(gcode);
    scroll_bar->setMaximum(gcode_view->getMaxScroll());
    scroll_bar->setValue(0);
}

void GCodeViewer::saveGCode()
{
    const QString &filename = QFileDialog::getSaveFileName(this, tr("Save GCode"), QString(), tr("GCode files (*.gcode)"));
    if (filename.isEmpty())
        return;

    QFile file(filename);
    file.open(QIODevice::WriteOnly | QIODevice::Text);
    file.write(gcode_view->toPlainText());
    file.close();
}

void GCodeViewer::loadGCode()
{
    const QString &filename = QFileDialog::getOpenFileName(this, tr("Load GCode"), QString(), tr("GCode files (*.gcode)"));
    if (filename.isEmpty())
        return;

    QFile file(filename);
    file.open(QIODevice::ReadOnly | QIODevice::Text);
	const QByteArray gcode = file.readAll();
	gcode_view->setPlainText(gcode);
    scroll_bar->setMaximum(gcode_view->getMaxScroll());
    scroll_bar->setValue(0);
    file.close();

	Mesh *mesh = new Mesh;
	const std::pair<Vec3, Vec3> &bbox = GCoder::instance()->parseGCode(gcode);
	mesh->setBoundingBox(bbox);
	Viewer::instance()->setMesh(mesh);
	Viewer::instance()->updateGL();
}
